@@ -24,6 +24,7 @@ class SpinIO(object):
         spinat,
         charges,
         index_spin,
+        orbital_names={},
         colinear=True,
         distance_dict=None,
         exchange_Jdict=None,
@@ -71,7 +72,8 @@ class SpinIO(object):
         if self.colinear and self.spinat != []:
             self.magmoms = np.array(self.spinat)[:, 2]
         self.charges = charges
-        self.distance_dict = distance_dict  #: A dictionary of distances, the keys are (i,j, R), where i and j are spin index and R is the cell index, a tuple of three integers.
+        #: A dictionary of distances, the keys are (i,j, R), where i and j are spin index and R is the cell index, a tuple of three integers.
+        self.distance_dict = distance_dict
         self.ind_atoms = {}  #: The index of atom for each spin.
         for iatom, ispin in enumerate(self.index_spin):
             if ispin >= 0:
@@ -79,7 +81,8 @@ class SpinIO(object):
 
         if exchange_Jdict is not None:
             self.has_exchange = True  #: whether there is isotropic exchange
-            self.exchange_Jdict = exchange_Jdict  #: The dictionary of :math:`J_{ij}(R)`, the keys are (i,j, R), where R is a tuple, and the value is the isotropic exchange
+            #: The dictionary of :math:`J_{ij}(R)`, the keys are (i,j, R), where R is a tuple, and the value is the isotropic exchange
+            self.exchange_Jdict = exchange_Jdict
         else:
             self.has_exchange = False
             self.exchange_Jdict = None
@@ -90,14 +93,16 @@ class SpinIO(object):
 
         if dmi_ddict is not None:
             self.has_dmi = True  #: Whether there is DMI.
-            self.dmi_ddict = dmi_ddict  #: The dictionary of DMI. the key is the same as exchange_Jdict, the values are 3-d vectors (Dx, Dy, Dz).
+            #: The dictionary of DMI. the key is the same as exchange_Jdict, the values are 3-d vectors (Dx, Dy, Dz).
+            self.dmi_ddict = dmi_ddict
         else:
             self.has_dmi = False
             self.dmi_ddict = None
 
         if Jani_dict is not None:
             self.has_bilinear = True  #: Whether there is anisotropic exchange term
-            self.Jani_dict = Jani_dict  #: The dictionary of anisotropic exchange. The vlaues are matrices of shape (3,3).
+            #: The dictionary of anisotropic exchange. The vlaues are matrices of shape (3,3).
+            self.Jani_dict = Jani_dict
         else:
             self.has_bilinear = False
             self.Jani_dict = None
@@ -145,13 +150,14 @@ class SpinIO(object):
 
         self.write_experimental = write_experimental
 
-        now=datetime.now()
+        now = datetime.now()
         self.description = f"""Exchange parameters generated by TB2J {__version__}
 Generation time: {now.strftime("%y/%m/%d %H:%M:%S")}
 """
         if description is not None:
             self.description += description
 
+        self.orbital_names = orbital_names
 
     def write_pickle(self, path='TB2J_results', fname='TB2J.pickle'):
         if not os.path.exists(path):
@@ -160,8 +166,8 @@ Generation time: {now.strftime("%y/%m/%d %H:%M:%S")}
         with open(fname, 'wb') as myfile:
             try:
                 pickle.dump(self.__dict__, myfile)
-            except:
-                print("Pickle not written.")
+            except Exception as ex:
+                print(f"Pickle not written due to {ex}")
 
     @classmethod
     def load_pickle(cls, path='TB2J_resutls', fname='TB2J.pickle'):
@@ -188,7 +194,7 @@ Generation time: {now.strftime("%y/%m/%d %H:%M:%S")}
         from TB2J.io_exchange.io_multibinit import write_multibinit
         write_multibinit(self, path=path)
 
-    def write_Jq(self, kmesh, path,**kwargs):
+    def write_Jq(self, kmesh, path, **kwargs):
         from TB2J.spinham.spin_api import SpinModel
         from TB2J.io_exchange.io_txt import write_Jq_info
         m = SpinModel(fname=os.path.join(path, 'Multibinit', 'exchange.xml'))
